@@ -16,6 +16,7 @@ use net::{ToSocketAddrs, SocketAddr, Shutdown};
 use sys_common::net as net_imp;
 use sys_common::{AsInner, FromInner, IntoInner};
 use std::time::Duration;
+use {SOCKET, INVALID_SOCKET};
 
 /// A TCP stream between a local and a remote socket.
 ///
@@ -112,6 +113,13 @@ impl TcpSocket {
         super::each_addr(addr, net_imp::TcpSocket::connect).map(TcpSocket)
     }
 
+    // provide socket use new_v4() or new_v6()
+    pub fn s_connect<A: ToSocketAddrs>(&self, addr: A) -> io::Result<()> {
+        super::each_addr(addr, |addr| {
+            self.0.s_connect(addr)
+        })
+    }
+
     /// Opens a TCP connection to a remote host with a timeout.
     ///
     /// Unlike `connect`, `connect_timeout` takes a single [`SocketAddr`] since
@@ -129,6 +137,11 @@ impl TcpSocket {
         net_imp::TcpSocket::connect_timeout(addr, timeout).map(TcpSocket)
     }
 
+    // provide socket use new_v4() or new_v6()
+    pub fn s_connect_timeout(&self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
+        self.0.s_connect_timeout(addr, timeout)
+    }
+
     /// Opens a TCP connection to a remote host with asyn.
     /// It will return immediately but the stream status is not ok, 
     /// when you want to use it, must call check_ready to check ok
@@ -136,18 +149,25 @@ impl TcpSocket {
         net_imp::TcpSocket::connect_asyn(addr).map(TcpSocket)
     }
 
-    /// return -1 is not a valid socket
-    /// return > 0 when the socket is ok
-    pub fn get_socket_fd(&self) -> i32 {
-        self.0.get_socket_fd()
+    // provide socket use new_v4() or new_v6()
+    pub fn s_connect_asyn(&self, addr: &SocketAddr) -> io::Result<()> {
+        self.0.s_connect_asyn(addr)
+    }
+
+    pub fn as_raw_socket(&self) -> SOCKET {
+        self.0.as_raw_socket()
     }
 
     /// new by the socket fd, it will always set ready ok
     /// it will not a valid socket, when fd set -1
-    pub fn new_by_fd(fd: i32) -> io::Result<TcpSocket> {
+    pub fn new_by_fd(fd: SOCKET) -> io::Result<TcpSocket> {
         net_imp::TcpSocket::new_by_fd(fd).map(TcpSocket)
     }
 
+    pub fn new_invalid() -> io::Result<TcpSocket> {
+        net_imp::TcpSocket::new_by_fd(INVALID_SOCKET).map(TcpSocket)
+    }
+
     /// check the socket is valid.
     pub fn is_valid(&self) -> bool {
         self.0.is_valid()
@@ -159,12 +179,20 @@ impl TcpSocket {
     }
 
     /// set socket ready status manual.
-    pub fn set_ready(&mut self, ready: bool) {
+    pub fn set_ready(&self, ready: bool) {
         self.0.set_ready(ready);
     }
 
+    pub fn is_close(&self) -> bool {
+        self.0.is_close()
+    }
+
+    pub fn close(&self) {
+        self.0.close();
+    }
+
     /// check socket ready status, when you connect remote host by asyn.
-    pub fn check_ready(&mut self) -> io::Result<bool> {
+    pub fn check_ready(&self) -> io::Result<bool> {
         self.0.check_ready()
     }
 
@@ -184,6 +212,10 @@ impl TcpSocket {
         self.0.peer_addr()
     }
 
+    pub fn set_peer_addr(&mut self, addr: SocketAddr) {
+        self.0.set_peer_addr(addr);
+    }
+
     /// Returns the socket address of the local half of this TCP connection.
     ///
     /// # Examples
@@ -200,6 +232,10 @@ impl TcpSocket {
         self.0.socket_addr()
     }
 
+    pub fn set_local_addr(&mut self, addr: SocketAddr) {
+        self.0.set_socket_addr(addr);
+    }
+
     /// Shuts down the read, write, or both halves of this connection.
     ///
     /// This function will cause all pending and future I/O on the specified
@@ -488,11 +524,11 @@ impl TcpSocket {
     /// ```no_run
     /// use psocket::TcpSocket;
     ///
-    /// let mut stream = TcpSocket::connect("127.0.0.1:8080")
+    /// let stream = TcpSocket::connect("127.0.0.1:8080")
     ///                        .expect("Couldn't connect to the server...");
     /// stream.set_nonblocking(true).expect("set_nonblocking call failed");
     /// ```
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
         self.0.set_nonblocking(nonblocking)
     }
 
@@ -502,7 +538,7 @@ impl TcpSocket {
     /// ```no_run
     /// use psocket::TcpSocket;
     ///
-    /// let mut stream = TcpSocket::connect("127.0.0.1:8080")
+    /// let stream = TcpSocket::connect("127.0.0.1:8080")
     ///                        .expect("Couldn't connect to the server...");
     /// stream.set_nonblocking(true).expect("set_nonblocking call failed");
     /// assert!(stream.is_nonblocking());
@@ -692,6 +728,20 @@ impl TcpSocket {
     pub fn unlink(self) -> io::Result<()> {
         self.0.unlink()
     }
+
+
+    /// Constructs a new TcpBuilder with the `AF_INET` domain, the `SOCK_STREAM`
+    /// type, and with a protocol argument of 0.
+    pub fn new_v4() -> io::Result<TcpSocket> {
+        net_imp::TcpSocket::new_v4().map(TcpSocket)
+    }
+
+    /// Constructs a new TcpBuilder with the `AF_INET6` domain, the `SOCK_STREAM`
+    /// type, and with a protocol argument of 0.
+    pub fn new_v6() -> io::Result<TcpSocket> {
+        net_imp::TcpSocket::new_v6().map(TcpSocket)
+    }
+
 }
 
 impl Clone for TcpSocket {
@@ -740,6 +790,7 @@ impl<'a> Iterator for Incoming<'a> {
     }
 }
 
+
 #[cfg(all(test, not(target_os = "emscripten")))]
 mod tests {
     use std::io::ErrorKind;
@@ -1447,7 +1498,7 @@ mod tests {
     #[test]
     fn set_nonblocking() {
         let addr = next_test_ip4();
-        let mut listener = t!(TcpSocket::bind(&addr));
+        let listener = t!(TcpSocket::bind(&addr));
 
         t!(listener.set_nonblocking(true));
         t!(listener.set_nonblocking(false));
