@@ -14,6 +14,7 @@ use net::{ToSocketAddrs, SocketAddr, Ipv4Addr, Ipv6Addr};
 use sys_common::net as net_imp;
 use sys_common::{AsInner, FromInner, IntoInner};
 use std::time::Duration;
+use {SOCKET};
 
 /// A UDP socket.
 ///
@@ -699,12 +700,16 @@ impl UdpSocket {
     /// ```no_run
     /// use psocket::UdpSocket;
     ///
-    /// let mut socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
+    /// let socket = UdpSocket::bind("127.0.0.1:34254").expect("couldn't bind to address");
     /// socket.set_nonblocking(true).expect("set_nonblocking call failed");
     /// ```
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
         self.0.set_nonblocking(nonblocking)
     }
+
+    pub fn as_raw_socket(&self) -> SOCKET {
+        self.0.as_raw_socket()
+    }
 }
 
 impl AsInner<net_imp::UdpSocket> for UdpSocket {
@@ -1034,7 +1039,7 @@ mod tests {
     #[test]
     fn set_nonblocking() {
         each_ip(&mut |addr, _| {
-            let mut socket = t!(UdpSocket::bind(&addr));
+            let socket = t!(UdpSocket::bind(&addr));
 
             t!(socket.set_nonblocking(true));
             t!(socket.set_nonblocking(false));
