@@ -10,7 +10,7 @@
 
 use std::ffi::CStr;
 use std::io;
-use libc::{self, c_int, c_void, size_t, sockaddr, socklen_t, EAI_SYSTEM, MSG_PEEK, accept4, ssize_t};
+use libc::{self, c_int, c_void, size_t, sockaddr, socklen_t, EAI_SYSTEM, MSG_PEEK, ssize_t};
 use std::mem;
 use net::{SocketAddr, Shutdown};
 use std::str;
@@ -18,6 +18,7 @@ use sys_common::{AsInner, FromInner, IntoInner};
 use sys_common::net::{getsockopt, setsockopt, sockaddr_to_addr};
 use std::time::{Duration, Instant};
 use std::cmp;
+use std::cell::Cell;
 
 pub use sys::{cvt, cvt_r};
 pub extern crate libc as netc;
@@ -25,6 +26,7 @@ pub extern crate libc as netc;
 pub type wrlen_t = size_t;
 
 pub const INVALID_SOCKET: c_int = -1;
+pub type SOCKET = c_int;
 
 fn max_len() -> usize {
     if cfg!(target_os = "macos") {
@@ -34,9 +36,9 @@ fn max_len() -> usize {
     }
 }
 
-extern {
-    fn gnu_get_libc_version() -> *const libc::c_char;
-}
+
+#[cfg(target_os = "linux")]
+use libc::accept4;
 
 // See below for the usage of SOCK_CLOEXEC, but this constant is only defined on
 // Linux currently (e.g. support doesn't exist on other platforms). In order to
@@ -56,8 +58,9 @@ use libc::SO_NOSIGPIPE;
 
 pub struct Socket {
     socket: c_int,
-    ready: bool,
-    nonblocking: bool,
+    ready: Cell<bool>,
+    nonblocking: Cell<bool>,
+    closed: Cell<bool>,
 }
 
 pub fn init() {}
@@ -91,16 +94,18 @@ impl Socket {
     pub fn new_by_fd(fd: c_int) -> Socket {
         Socket {
             socket: fd,
-            ready: false,
-            nonblocking: false,
+            ready: Cell::new(false),
+            nonblocking: Cell::new(false),
+            closed: Cell::new(false),
         }
     }
 
     pub fn new_ready_fd(fd: c_int) -> Socket {
         Socket {
             socket: fd,
-            ready: true,
-            nonblocking: false,
+            ready: Cell::new(true),
+            nonblocking: Cell::new(false),
+            closed: Cell::new(false),
         }
     }
 
@@ -130,6 +135,14 @@ impl Socket {
         }
     }
 
+    pub fn new_v4() -> io::Result<Socket> {
+        Self::new_raw(libc::AF_INET, libc::SOCK_STREAM)
+    }
+
+    pub fn new_v6() -> io::Result<Socket> {
+        Self::new_raw(libc::AF_INET6, libc::SOCK_STREAM)
+    }
+
     pub fn new_pair(fam: c_int, ty: c_int) -> io::Result<(Socket, Socket)> {
         unsafe {
             let mut fds = [0, 0];
@@ -154,7 +167,7 @@ impl Socket {
         }
     }
 
-    pub fn connect_timeout(&mut self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
+    pub fn connect_timeout(&self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
         self.set_nonblocking(true)?;
         let r = unsafe {
             let (addrp, len) = addr.into_inner();
@@ -223,7 +236,7 @@ impl Socket {
         }
     }
 
-    pub fn connect_asyn(&mut self, addr: &SocketAddr) -> io::Result<()> {
+    pub fn connect_asyn(&self, addr: &SocketAddr) -> io::Result<()> {
         self.set_nonblocking(true)?;
         let r = unsafe {
             let (addrp, len) = addr.into_inner();
@@ -241,20 +254,12 @@ impl Socket {
         }
     }
 
-    pub fn get_socket_fd(&self) -> i32 {
-        self.socket as i32
-    }
-
-    pub fn new_out_fd(fd: i32) -> Socket {
-        let socket = if fd == -1 {
-            INVALID_SOCKET
-        } else {
-            fd as c_int
-        };
+    pub fn new_out_fd(fd: SOCKET) -> Socket {
         Socket {
-            socket: socket,
-            ready: true,
-            nonblocking: false,
+            socket: fd,
+            ready: Cell::new(true),
+            nonblocking: Cell::new(false),
+            closed: Cell::new(false),
         }
     }
 
@@ -263,24 +268,24 @@ impl Socket {
     }
 
     pub fn is_ready(&self) -> bool {
-        self.ready
+        self.ready.get()
     }
 
-    pub fn set_ready(&mut self, ready: bool) {
-        self.ready = ready;
+    pub fn set_ready(&self, ready: bool) {
+        self.ready.set(ready);
     }
 
     pub fn ensure_ready(&self) -> io::Result<()> {
-        if !self.ready {
+        if !self.ready.get() {
             return Err(io::Error::new(io::ErrorKind::NotConnected,
                                       "current socket is not ready"));
         }
         Ok(())
     }
 
-    pub fn check_ready(&mut self) -> io::Result<bool> {
-        if self.ready {
-            return Ok(self.ready);
+    pub fn check_ready(&self) -> io::Result<bool> {
+        if self.ready.get() {
+            return Ok(self.ready.get());
         }
 
         let mut pollfd = libc::pollfd {
@@ -309,11 +314,11 @@ impl Socket {
                     return Err(e);
                 }
 
-                self.ready = true;
+                self.ready.set(true);
             }
         }
 
-        Ok(self.ready)
+        Ok(self.ready.get())
     }
 
     pub fn set_liner(&self, enable: bool, time: u16) -> io::Result<()> {
@@ -360,16 +365,16 @@ impl Socket {
         // atomically set the CLOEXEC flag is to use the `accept4` syscall on
         // Linux. This was added in 2.6.28, however, and because we support
         // 2.6.18 we must detect this support dynamically.
-        if cfg!(target_os = "linux") {
-            let res = cvt_r(|| unsafe {
-                accept4(self.socket, storage, len, SOCK_CLOEXEC)
-            });
-            match res {
-                Ok(fd) => return Ok(Socket::new_ready_fd(fd)),
-                Err(ref e) if e.raw_os_error() == Some(libc::ENOSYS) => {}
-                Err(e) => return Err(e),
-            }
-        }
+        // if cfg!(target_os = "linux") {
+        //     let res = cvt_r(|| unsafe {
+        //         accept4(self.socket, storage, len, SOCK_CLOEXEC)
+        //     });
+        //     match res {
+        //         Ok(fd) => return Ok(Socket::new_ready_fd(fd)),
+        //         Err(ref e) if e.raw_os_error() == Some(libc::ENOSYS) => {}
+        //         Err(e) => return Err(e),
+        //     }
+        // }
 
         let fd = cvt_r(|| unsafe {
             libc::accept(self.socket, storage, len)
@@ -505,15 +510,15 @@ impl Socket {
         Ok(raw != 0)
     }
 
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
         let mut nonblocking = nonblocking as libc::c_int;
         cvt(unsafe { libc::ioctl(*self.as_inner(), libc::FIONBIO, &mut nonblocking) })?;
-        self.nonblocking = nonblocking == 1;
+        self.nonblocking.set(nonblocking == 1);
         Ok(())
     }
 
     pub fn is_nonblocking(&self) -> bool {
-        self.nonblocking
+        self.nonblocking.get()
     }
 
 
@@ -533,6 +538,16 @@ impl Socket {
         }
     }
 
+    pub fn close(&self) {
+        let _ = unsafe { libc::close(self.socket) };
+        self.closed.set(true);
+    }
+
+    pub fn is_close(&self) -> bool {
+        self.closed.get()
+    }
+
+
     pub fn unlink(mut self) -> c_int {
         let sock = self.socket;
         self.socket = INVALID_SOCKET;
@@ -542,7 +557,7 @@ impl Socket {
 
 impl Drop for Socket {
     fn drop(&mut self) {
-        if self.socket == INVALID_SOCKET {
+        if self.socket == INVALID_SOCKET || self.is_close() {
             return;
         }
         let _ = unsafe { libc::close(self.socket) };
@@ -553,8 +568,9 @@ impl Clone for Socket {
     fn clone(&self) -> Socket {
         Socket {
             socket: self.socket,
-            ready: self.ready,
-            nonblocking: self.nonblocking,
+            ready: self.ready.clone(),
+            nonblocking: self.nonblocking.clone(),
+            closed: self.closed.clone(),
         }
     }
 }
@@ -568,52 +584,7 @@ impl FromInner<c_int> for Socket {
 }
 
 impl IntoInner<c_int> for Socket {
-    fn into_inner(mut self) -> c_int {
+    fn into_inner(self) -> c_int {
         self.unlink()
     }
 }
-
-// In versions of glibc prior to 2.26, there's a bug where the DNS resolver
-// will cache the contents of /etc/resolv.conf, so changes to that file on disk
-// can be ignored by a long-running program. That can break DNS lookups on e.g.
-// laptops where the network comes and goes. See
-// https://sourceware.org/bugzilla/show_bug.cgi?id=984. Note however that some
-// distros including Debian have patched glibc to fix this for a long time.
-//
-// A workaround for this bug is to call the res_init libc function, to clear
-// the cached configs. Unfortunately, while we believe glibc's implementation
-// of res_init is thread-safe, we know that other implementations are not
-// (https://github.com/rust-lang/rust/issues/43592). Code here in libstd could
-// try to synchronize its res_init calls with a Mutex, but that wouldn't
-// protect programs that call into libc in other ways. So instead of calling
-// res_init unconditionally, we call it only when we detect we're linking
-// against glibc version < 2.26. (That is, when we both know its needed and
-// believe it's thread-safe).
-pub fn res_init_if_glibc_before_2_26() -> io::Result<()> {
-    // If the version fails to parse, we treat it the same as "not glibc".
-    if let Some(Ok(version_str)) = glibc_version_cstr().map(CStr::to_str) {
-        if let Some(version) = parse_glibc_version(version_str) {
-            if version < (2, 26) {
-                let ret = unsafe { libc::res_init() };
-                if ret != 0 {
-                    return Err(io::Error::last_os_error());
-                }
-            }
-        }
-    }
-    Ok(())
-}
-
-fn glibc_version_cstr() -> Option<&'static CStr> {
-    unsafe { Some(CStr::from_ptr(gnu_get_libc_version())) }
-}
-
-// Returns Some((major, minor)) if the string is a valid "x.y" version,
-// ignoring any extra dot-separated parts. Otherwise return None.
-fn parse_glibc_version(version: &str) -> Option<(usize, usize)> {
-    let mut parsed_ints = version.split(".").map(str::parse::<usize>).fuse();
-    match (parsed_ints.next(), parsed_ints.next()) {
-        (Some(Ok(major)), Some(Ok(minor))) => Some((major, minor)),
-        _ => None
-    }
-}
