@@ -20,6 +20,7 @@ use sys;
 use sys_common::{AsInner, FromInner, IntoInner};
 use sys_common::net;
 use std::time::Duration;
+use std::cell::Cell;
 
 pub type wrlen_t = i32;
 
@@ -33,8 +34,9 @@ pub mod netc {
 
 pub struct Socket {
     socket: c::SOCKET,
-    ready: bool,
-    nonblocking: bool,
+    ready: Cell<bool>,
+    closed: Cell<bool>,
+    nonblocking: Cell<bool>,
 }
 
 /// Checks whether the Windows socket interface has been started already, and
@@ -112,8 +114,45 @@ impl Socket {
                 c::INVALID_SOCKET => Err(last_error()),
                 n => Ok(Socket {
                     socket: n,
-                    ready: false,
-                    nonblocking: false,
+                    ready: Cell::new(false),
+                    closed: Cell::new(false),
+                    nonblocking: Cell::new(false),
+                }),
+            }
+        }?;
+        socket.set_no_inherit()?;
+        Ok(socket)
+    }
+    
+    pub fn new_v4() -> io::Result<Socket> {
+        init();
+        let socket = unsafe {
+            match c::WSASocketW(c::AF_INET, c::SOCK_STREAM, 0, ptr::null_mut(), 0,
+                                c::WSA_FLAG_OVERLAPPED) {
+                c::INVALID_SOCKET => Err(last_error()),
+                n => Ok(Socket {
+                    socket: n,
+                    ready: Cell::new(false),
+                    closed: Cell::new(false),
+                    nonblocking: Cell::new(false),
+                }),
+            }
+        }?;
+        socket.set_no_inherit()?;
+        Ok(socket)
+    }
+
+    pub fn new_v6() -> io::Result<Socket> {
+        init();
+        let socket = unsafe {
+            match c::WSASocketW(c::AF_INET6, c::SOCK_STREAM, 0, ptr::null_mut(), 0,
+                                c::WSA_FLAG_OVERLAPPED) {
+                c::INVALID_SOCKET => Err(last_error()),
+                n => Ok(Socket {
+                    socket: n,
+                    ready: Cell::new(false),
+                    closed: Cell::new(false),
+                    nonblocking: Cell::new(false),
                 }),
             }
         }?;
@@ -126,24 +165,24 @@ impl Socket {
     }
 
     pub fn is_ready(&self) -> bool {
-        self.ready
+        self.ready.get()
     }
 
-    pub fn set_ready(&mut self, ready: bool) {
-        self.ready = ready;
+    pub fn set_ready(&self, ready: bool) {
+        self.ready.set(ready);
     }
 
     pub fn ensure_ready(&self) -> io::Result<()> {
-        if !self.ready {
+        if !self.ready.get() {
             return Err(io::Error::new(io::ErrorKind::NotConnected,
                                       "current socket is not ready"));
         }
         Ok(())
     }
 
-    pub fn check_ready(&mut self) -> io::Result<bool> {
-        if self.ready {
-            return Ok(self.ready);
+    pub fn check_ready(&self) -> io::Result<bool> {
+        if self.ready.get() {
+            return Ok(self.ready.get());
         }
         let timeout = c::timeval {
             tv_sec: 0 as c_long,
@@ -168,13 +207,13 @@ impl Socket {
                     return Err(e);
                 }
             }
-            self.ready = true
+            self.ready.set(true);
         }
 
-        Ok(self.ready)
+        Ok(self.ready.get())
     }
 
-    pub fn connect_timeout(&mut self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
+    pub fn connect_timeout(&self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
         self.set_nonblocking(true)?;
         let r = unsafe {
             let (addrp, len) = addr.into_inner();
@@ -229,7 +268,7 @@ impl Socket {
     }
 
 
-    pub fn connect_asyn(&mut self, addr: &SocketAddr) -> io::Result<()> {
+    pub fn connect_asyn(&self, addr: &SocketAddr) -> io::Result<()> {
         self.set_nonblocking(true)?;
         let r = unsafe {
             let (addrp, len) = addr.into_inner();
@@ -246,23 +285,12 @@ impl Socket {
         }
     }
 
-    pub fn get_socket_fd(&self) -> i32 {
-        if self.socket == c::INVALID_SOCKET {
-            return -1;
-        }
-        self.socket as i32
-    }
-
-    pub fn new_out_fd(fd: i32) -> Socket {
-        let socket = if fd == -1 {
-            c::INVALID_SOCKET
-        } else {
-            fd as c::SOCKET
-        };
+    pub fn new_out_fd(fd: c::SOCKET) -> Socket {
         Socket {
-            socket: socket,
-            ready: true,
-            nonblocking: false,
+            socket: fd,
+            ready: Cell::new(true),
+            closed: Cell::new(false),
+            nonblocking: Cell::new(false),
         }
     }
 
@@ -273,8 +301,9 @@ impl Socket {
                 c::INVALID_SOCKET => Err(last_error()),
                 n => Ok(Socket {
                     socket: n,
-                    ready: true,
-                    nonblocking: false,
+                    ready: Cell::new(true),
+                    closed: Cell::new(false),
+                    nonblocking: Cell::new(false),
                 }),
             }
         }?;
@@ -296,8 +325,9 @@ impl Socket {
                 c::INVALID_SOCKET => Err(last_error()),
                 n => Ok(Socket {
                     socket: n,
-                    ready: true,
-                    nonblocking: false,
+                    ready: Cell::new(true),
+                    closed: Cell::new(false),
+                    nonblocking: Cell::new(false),
                 }),
             }
         }?;
@@ -407,11 +437,11 @@ impl Socket {
         Ok(())
     }
 
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
         let mut nonblocking = nonblocking as c_ulong;
         let r = unsafe { c::ioctlsocket(self.socket, c::FIONBIO as c_int, &mut nonblocking) };
         if r == 0 {
-            self.nonblocking = nonblocking == 1;
+            self.nonblocking.set(nonblocking == 1);
             Ok(())
         } else {
             Err(io::Error::last_os_error())
@@ -419,7 +449,7 @@ impl Socket {
     }
 
     pub fn is_nonblocking(&self) -> bool {
-        self.nonblocking
+        self.nonblocking.get()
     }
 
     pub fn set_liner(&self, enable: bool, time: u16) -> io::Result<()> {
@@ -476,6 +506,15 @@ impl Socket {
         }
     }
 
+    pub fn close(&self) {
+        let _ = unsafe { c::closesocket(self.socket) };
+        self.closed.set(true);
+    }
+
+    pub fn is_close(&self) -> bool {
+        self.closed.get()
+    }
+
     pub fn unlink(mut self) -> c::SOCKET {
         let sock = self.socket;
         self.socket = c::INVALID_SOCKET;
@@ -492,7 +531,7 @@ impl<'a> Read for &'a Socket {
 
 impl Drop for Socket {
     fn drop(&mut self) {
-        if self.socket == c::INVALID_SOCKET {
+        if self.socket == c::INVALID_SOCKET || self.closed.get() {
             return;
         }
         let _ = unsafe { c::closesocket(self.socket) };
@@ -503,8 +542,9 @@ impl Clone for Socket {
     fn clone(&self) -> Socket {
         Socket {
             socket: self.socket,
-            ready: self.ready,
-            nonblocking: self.nonblocking,
+            ready: self.ready.clone(),
+            closed: self.closed.clone(),
+            nonblocking: self.nonblocking.clone(),
         }
     }
 }
@@ -516,8 +556,9 @@ impl AsInner<c::SOCKET> for Socket {
 impl FromInner<c::SOCKET> for Socket {
     fn from_inner(sock: c::SOCKET) -> Socket { Socket {
         socket: sock,
-        ready: true,
-        nonblocking: false,
+        ready: Cell::new(true),
+        closed: Cell::new(false),
+        nonblocking: Cell::new(false),
     }}
 }
 
