@@ -20,6 +20,7 @@ use sys::net::{cvt, cvt_r, cvt_gai, Socket, init, wrlen_t};
 use sys::net::netc as c;
 use sys_common::{AsInner, FromInner, IntoInner};
 use std::time::Duration;
+use {SOCKET};
 
 #[cfg(any(target_os = "dragonfly", target_os = "freebsd",
           target_os = "ios", target_os = "macos",
@@ -173,21 +174,6 @@ pub fn lookup_host(host: &str) -> io::Result<LookupHost> {
             Ok(_) => {
                 Ok(LookupHost { original: res, cur: res })
             },
-            #[cfg(unix)]
-            Err(e) => {
-                // If we're running glibc prior to version 2.26, the lookup
-                // failure could be caused by caching a stale /etc/resolv.conf.
-                // We need to call libc::res_init() to clear the cache. But we
-                // shouldn't call it in on any other platform, because other
-                // res_init implementations aren't thread-safe. See
-                // https://github.com/rust-lang/rust/issues/41570 and
-                // https://github.com/rust-lang/rust/issues/43592.
-                use sys::net::res_init_if_glibc_before_2_26;
-                let _ = res_init_if_glibc_before_2_26();
-                Err(e)
-            },
-            // the cfg is needed here to avoid an "unreachable pattern" warning
-            #[cfg(not(unix))]
             Err(e) => Err(e),
         }
     }
@@ -199,43 +185,87 @@ pub fn lookup_host(host: &str) -> io::Result<LookupHost> {
 
 pub struct TcpSocket {
     inner: Socket,
+    local: Option<SocketAddr>,
+    remote: Option<SocketAddr>,
 }
 
 impl TcpSocket {
     pub fn connect(addr: &SocketAddr) -> io::Result<TcpSocket> {
         init();
 
-        let mut sock = Socket::new(addr, c::SOCK_STREAM)?;
+        let sock = Socket::new(addr, c::SOCK_STREAM)?;
         let (addrp, len) = addr.into_inner();
         cvt_r(|| unsafe { c::connect(*sock.as_inner(), addrp, len) })?;
         sock.set_ready(true);
-        Ok(TcpSocket { inner: sock })
+        Ok(TcpSocket { 
+            inner: sock,
+            remote: Some(addr.clone()),
+            local: None,
+        })
+    }
+
+    pub fn s_connect(&self, addr: &SocketAddr) -> io::Result<()> {
+        if self.is_ready() {
+            return Err(Error::new(ErrorKind::AlreadyExists, "the socket is already connect"))
+        }
+
+        let (addrp, len) = addr.into_inner();
+        cvt_r(|| unsafe { c::connect(*self.inner.as_inner(), addrp, len) })?;
+        self.set_ready(true);
+        Ok(())
     }
 
     pub fn connect_timeout(addr: &SocketAddr, timeout: Duration) -> io::Result<TcpSocket> {
         init();
 
-        let mut sock = Socket::new(addr, c::SOCK_STREAM)?;
+        let sock = Socket::new(addr, c::SOCK_STREAM)?;
         sock.connect_timeout(addr, timeout)?;
         sock.set_ready(true);
-        Ok(TcpSocket { inner: sock })
+        Ok(TcpSocket { 
+            inner: sock,
+            remote: Some(addr.clone()),
+            local: None,
+        })
+    }
+
+    pub fn s_connect_timeout(&self, addr: &SocketAddr, timeout: Duration) -> io::Result<()> {
+        if self.is_ready() {
+            return Err(Error::new(ErrorKind::AlreadyExists, "the socket is already connect"))
+        }
+
+        self.inner.connect_timeout(addr, timeout)?;
+        self.inner.set_ready(true);
+        Ok(())
     }
 
     pub fn connect_asyn(addr: &SocketAddr) -> io::Result<TcpSocket> {
         init();
 
-        let mut sock = Socket::new(addr, c::SOCK_STREAM)?;
+        let sock = Socket::new(addr, c::SOCK_STREAM)?;
         sock.connect_asyn(addr)?;
-        Ok(TcpSocket { inner: sock })
+        Ok(TcpSocket { 
+            inner: sock,
+            remote: Some(addr.clone()),
+            local: None,
+        })
     }
 
-    pub fn get_socket_fd(&self) -> i32 {
-        self.inner.get_socket_fd()
+    pub fn s_connect_asyn(&self, addr: &SocketAddr) -> io::Result<()> {
+        if self.is_ready() {
+            return Err(Error::new(ErrorKind::AlreadyExists, "the socket is already connect"))
+        }
+
+        self.inner.connect_asyn(addr)?;
+        Ok(())
     }
 
-    pub fn new_by_fd(fd: i32) -> io::Result<TcpSocket> {
+    pub fn new_by_fd(fd: SOCKET) -> io::Result<TcpSocket> {
         let sock = Socket::new_out_fd(fd);
-        Ok(TcpSocket { inner: sock })
+        Ok(TcpSocket { 
+            inner: sock,
+            remote: None,
+            local: None,
+        })
     }
 
     pub fn is_valid(&self) -> bool {
@@ -246,11 +276,19 @@ impl TcpSocket {
         self.inner.is_ready()
     }
 
-    pub fn set_ready(&mut self, ready: bool) {
+    pub fn set_ready(&self, ready: bool) {
         self.inner.set_ready(ready);
     }
 
-    pub fn check_ready(&mut self) -> io::Result<bool> {
+    pub fn is_close(&self) -> bool {
+        self.inner.is_close()
+    }
+
+    pub fn close(&self) {
+        self.inner.close();
+    }
+
+    pub fn check_ready(&self) -> io::Result<bool> {
         self.inner.check_ready()
     }
 
@@ -297,23 +335,41 @@ impl TcpSocket {
     }
 
     pub fn peer_addr(&self) -> io::Result<SocketAddr> {
+        if let Some(addr) = self.remote {
+            return Ok(addr.clone());
+        }
         sockname(|buf, len| unsafe {
             c::getpeername(*self.inner.as_inner(), buf, len)
         })
     }
 
+    pub fn set_peer_addr(&mut self, addr: SocketAddr) {
+        self.remote = Some(addr);
+    }
+
     pub fn socket_addr(&self) -> io::Result<SocketAddr> {
+        if let Some(addr) = self.local {
+            return Ok(addr.clone());
+        }
         sockname(|buf, len| unsafe {
             c::getsockname(*self.inner.as_inner(), buf, len)
         })
     }
 
+    pub fn set_socket_addr(&mut self, addr: SocketAddr) {
+        self.local = Some(addr);
+    }
+
     pub fn shutdown(&self, how: Shutdown) -> io::Result<()> {
         self.inner.shutdown(how)
     }
 
     pub fn duplicate(&self) -> io::Result<TcpSocket> {
-        self.inner.duplicate().map(|s| TcpSocket { inner: s })
+        self.inner.duplicate().map(|s| TcpSocket {
+            inner: s,
+            remote: self.remote.clone(),
+            local: self.local.clone(),
+        })
     }
 
     pub fn set_nodelay(&self, nodelay: bool) -> io::Result<()> {
@@ -337,7 +393,7 @@ impl TcpSocket {
         self.inner.take_error()
     }
 
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(&self, nonblocking: bool) -> io::Result<()> {
         self.inner.set_nonblocking(nonblocking)
     }
 
@@ -392,7 +448,11 @@ impl TcpSocket {
 
         // Start listening
         cvt(unsafe { c::listen(*sock.as_inner(), 128) })?;
-        Ok(TcpSocket { inner: sock })
+        Ok(TcpSocket { 
+            inner: sock,
+            remote: None,
+            local: Some(addr.clone()), 
+        })
     }
 
     pub fn accept(&self) -> io::Result<(TcpSocket, SocketAddr)> {
@@ -401,24 +461,58 @@ impl TcpSocket {
         let sock = self.inner.accept(&mut storage as *mut _ as *mut _,
                                      &mut len)?;
         let addr = sockaddr_to_addr(&storage, len as usize)?;
-        Ok((TcpSocket { inner: sock, }, addr))
+        Ok((TcpSocket { 
+            inner: sock,
+            remote: Some(addr),
+            local: self.local.clone(),
+        }, addr))
     }
 
     pub fn unlink(self) -> io::Result<()> {
         self.inner.unlink();
         Ok(())
     }
+
+    pub fn new_v4() -> io::Result<TcpSocket> {
+        let socket = Socket::new_v4()?;
+        Ok(TcpSocket { 
+            inner: socket,
+            remote: None,
+            local: None,
+        })
+    }
+
+    pub fn new_v6() -> io::Result<TcpSocket> {
+        let socket = Socket::new_v6()?;
+        Ok(TcpSocket { 
+            inner: socket,
+            remote: None,
+            local: None,
+        })
+    }
+
+    pub fn as_raw_socket(&self) -> SOCKET {
+        self.inner.as_inner().clone()
+    }
 }
 
 impl Clone for TcpSocket {
     fn clone(&self) -> TcpSocket {
-        TcpSocket { inner: self.inner.clone() }
+        TcpSocket { 
+            inner: self.inner.clone(),
+            remote: self.remote.clone(),
+            local: self.local.clone(), 
+        }
     }
 }
 
 impl FromInner<Socket> for TcpSocket {
     fn from_inner(socket: Socket) -> TcpSocket {
-        TcpSocket { inner: socket }
+        TcpSocket { 
+            inner: socket,
+            remote: None,
+            local: None, 
+        }
     }
 }
 
@@ -594,7 +688,7 @@ impl UdpSocket {
         self.inner.take_error()
     }
 
-    pub fn set_nonblocking(&mut self, nonblocking: bool) -> io::Result<()> {
+    pub fn set_nonblocking(& self, nonblocking: bool) -> io::Result<()> {
         self.inner.set_nonblocking(nonblocking)
     }
 
@@ -621,6 +715,10 @@ impl UdpSocket {
         let (addrp, len) = addr.into_inner();
         cvt_r(|| unsafe { c::connect(*self.inner.as_inner(), addrp, len) }).map(|_| ())
     }
+
+    pub fn as_raw_socket(&self) -> SOCKET {
+        self.inner.as_inner().clone()
+    }
 }
 
 impl FromInner<Socket> for UdpSocket {
